# compute_age.py
# author: Marko Borazio for ESS, TU Darmstadt
# Nov 2012
# description:
# age computation

import numpy as np
import results_tus as res

#load the dataset GTUS 2001/2002
dta = np.array(np.load('datasets/gtud.npy'), np.byte )
ref = np.array(np.load('datasets/ref.npy'))
dta = np.sum(dta,1)

#fill in the gaps: duplicate age where there is none
for i,h in enumerate(ref[:,3]):
	if h > 0:
		curr = h
	ref[i,3] = curr

#age bounderies
bound =np.array([10, 15, 20, 25, 30, 35, 40, 45, 50, 55, 60, 65, 70, 75],int)

hh=list(set(ref[:,0]))						# list of all households (HH)
conf = np.zeros((len(hh),11,11),int)	# confusion matrix: number of HH, 11 activities

#calculate histogram: 14 age groups, 11 activities
hst = np.zeros((14,11),int)		
for cnt, u in enumerate(dta):
	hst[np.digitize([ref[cnt,3]],bound)[0]-1] += np.histogram(u,bins=range(1,13))[0]
			
# i = HH counter
# hhid = household ID
for i, hhid in enumerate(hh):
	ind = np.where(ref[:,0] == hhid)
	hsta = np.array(hst)			# init histogram
	
	# adjust histogram per age-group
	for k,usr in enumerate(dta[ind]):
		age = np.int(ref[ind,3][:,k])
		hsta[np.digitize([age],bound)[0]-1] -= np.histogram(usr,bins=range(1,13))[0]
		
	# compare user activity label with the "most common activity" from histogram
	# usr = each user of the household
	for k, usr in enumerate(dta[ind]):
		age = np.int(ref[ind,3][:,k])
		
		pos = np.digitize([age],bound)[0]-1
		# fill confusion matrix
		conf[i,np.argmax(hsta[pos])] += np.histogram(usr,bins=range(1,13))[0]
			
# save the resulting confusion matrix
np.save('conf_age.npy',conf)
