# compute_prev_loc.py
# author: Marko Borazio for ESS, TU Darmstadt
# Dec 2012
# description:
# previous activity and location computation

import numpy as np
import time_use as tu

#load the dataset GTUS 2001/2002
dta = np.array(np.load('datasets/gtud.npy'), np.byte )
ref = np.array(np.load('datasets/ref.npy'),int)

hh = list(set(ref[:,0]))						# number of households
conf = np.zeros((len(hh),11,11),int)	# confusion matrix: 11 activities

# get histogram: 10 locations, 11 activities
# for location x: if prev. activity is y, performed activity is z
hst = np.zeros((10,11,11),int)
# ucnt = user counter		
# usr = location and activity for the user
for ucnt, usr in enumerate(dta):
	# tcnt = timer counter
	# act = previous activity performed
	for tcnt, act in enumerate(usr.T):
		# consider the case that only one activity occured in 24-hours
		# this was the case once in the dataset (no.128, sleeping the whole day)
		if len(set(np.sum(usr,0))) == 1:
			continue
		prev = np.array(tu.get_prev(np.sum(usr,0),tcnt),int)
		loc = np.nonzero(act)[0][0]
		hst[loc,prev-1,act[loc]-1] += 1
		

# i = HH counter
# hhid = household ID
for i, hhid in enumerate(hh):
	rw = np.zeros(dta.shape[0],bool)
	rw[ref[:,0] == hhid] = True
	hsta = np.array(hst)			# init histogram
	# ucnt = user counter
	# usr = user data
	for ucnt, usr in enumerate(dta[rw]):
		# tcnt = timer counter
		# act = previous activity performed
		for tcnt, act in enumerate(usr.T):
			if len(set(np.sum(usr,0))) == 1:
				continue
			prev = np.array(tu.get_prev(np.sum(usr,0),tcnt),int)
			loc = np.nonzero(act)[0][0]
			# adjust histogram per activity code
			hsta[loc,prev-1,act[loc]-1] -= 1
		
	for ucnt, usr in enumerate(dta[rw]):
		for tcnt, act in enumerate(usr.T):
			if len(set(np.sum(usr,0))) == 1:
				continue
			prev = np.array(tu.get_prev(np.sum(usr,0),tcnt),int)
			loc = np.nonzero(act)[0][0]
			# fill confusion matrix
			conf[i,np.argmax(hsta[loc,prev-1]),act[loc]-1] += 1 

#save results
np.save('conf_prev_loc.npy',conf)
