# compute_prev_loc_time.py
# author: Marko Borazio for ESS, TU Darmstadt
# Dec 2012
# description:
# time, location and previous activity computation

import numpy as np
import itertools
import time_use as tu

#load the dataset GTUS 2001/2002
dta = np.array(np.load('datasets/gtud.npy'), np.byte )
ref = np.array(np.load('datasets/ref.npy'),int)

# number of households
hh=list(set(ref[:,0]))						
# confusion matrix: no. of HH, 11 activities
conf = np.zeros((len(hh),11,11),int)	

# get histogram: 144 time-slots, 10 locations, 11 activities
hst = np.zeros((144,10,11,11),int)

# ucnt = user counter		
# u = time-slot and activity of the user
for ucnt, usr in enumerate(dta):
	# tcnt = counter for time-slot
	# act = performed activity at time-slot tcnt
	for tcnt, act in enumerate(usr.T):
		# consider the case that only one activity occured in 24-hours
		# this was the case once in the dataset (no.128, sleeping the whole day)
		if len(set(np.sum(usr,0))) == 1:
			continue
		# get previous activity
		prev = np.array(tu.get_prev(np.sum(usr,0),tcnt),int)
		# loc = location where activity appeared
		loc = np.nonzero(act)[0][0]
		hst[tcnt,loc,prev-1,act[loc]-1] += 1
		
# i = HH counter
# hhid = household ID
for i, hhid in enumerate(hh):
	rw = np.zeros(dta.shape[0],bool)
	rw[ref[:,0] == hhid] = True
	hsta = np.array(hst)			# init histogram
	
	# ucnt = user counter		
	# usr = time-slot and activity of the user
	for ucnt, usr in enumerate(dta[rw]):
		# tcnt = counter for time-slot
		# act = performed activity at time-slot tcnt
		for tcnt, act in enumerate(usr.T):
			if len(set(np.sum(usr,0))) == 1:
				continue
			# get previous activity
			prev = np.array(tu.get_prev(np.sum(usr,0),tcnt),int)
			# loc = location where activity appeared
			loc = np.nonzero(act)[0][0]
			hst[tcnt,loc,prev-1,act[loc]-1] -= 1
	
	for ucnt, usr in enumerate(dta[rw]):
		# tcnt = counter for time-slot
		# act = performed activity at time-slot tcnt
		for tcnt, act in enumerate(usr.T):
			if len(set(np.sum(usr,0))) == 1:
				continue
			# get previous activity
			prev = np.array(tu.get_prev(np.sum(usr,0),tcnt),int)
			# loc = location where activity appeared
			loc = np.nonzero(act)[0][0]
			conf[i,np.argmax(hsta[tcnt,loc,prev-1]),act[loc]-1] += 1 
		
# save results	
np.save('conf_prev_loc_time.npy',conf)
