# compute_time_prev.py
# author: Marko Borazio for ESS, TU Darmstadt
# Dec 2012
# description:
# time and previous activity computation

import numpy as np
import time_use as tu
import itertools

#load the dataset GTUS 2001/2002
dta = np.array(np.load('datasets/gtud.npy'), np.byte )
ref = np.array(np.load('datasets/ref.npy'),int)

#get rid of location
dta = np.sum(dta,1)

hh=list(set(ref[:,0]))						# number of households
conf = np.zeros((len(hh),11,11),int)	# confusion matrix: 11 activities

# for time-slot x: if prev. activity is y, performed activity is z
hst = np.zeros((144,11,11),int)

# get histogram: 144 time-slots, 11 activities
# ucnt = user counter		
# u = time-slot and activity of the user
for ucnt, usr in enumerate(dta):
	if len(set(usr)) == 1:
		continue
	# tcnt = counter for time-slot
	# act = performed activity at time-slot tcnt
	for tcnt, act in enumerate(usr):
		#check previous activity
		prev = np.array(tu.get_prev(usr,tcnt),int)
		hst[tcnt,prev-1,usr[tcnt]-1] += 1
		
# i = HH counter
# hhid = household ID
for i, hhid in enumerate(hh):
	rw = np.zeros(dta.shape[0],bool)
	rw[ref[:,0] == hhid] = True
	hsta = np.array(hst)			# init histogram
	# ucnt = user counter
	# usr = user data
	for ucnt, usr in enumerate(dta[rw]):
		if len(set(usr)) == 1:
			continue
		# adjust histogram per activity code
		for tcnt, act in enumerate(usr):
			prev = np.array(tu.get_prev(usr,tcnt),int)
			hsta[tcnt,prev-1,usr[tcnt]-1] -= 1
			
	for ucnt, usr in enumerate(dta[rw]):
		if len(set(usr)) == 1:
			continue
		for tcnt, act in enumerate(usr):
			prev = np.array(tu.get_prev(usr,tcnt),int)
			# fill confusion matrix
			conf[i,np.argmax(hsta[tcnt,prev-1,:]),usr[tcnt]-1] += 1 

#save results	
np.save('conf_time_prev.npy',conf)
